@@ -121,6 +121,10 @@
 #define GOTSHOTGUN	"You got the shotgun!"
 #define GOTSHOTGUN2	"You got the super shotgun!"
 
+// [NS] Beta pickups.
+#define BETA_BONUS3	"Picked up an evil sceptre."
+#define BETA_BONUS4	"Picked up an unholy bible."
+
 //
 // P_Doors.C
 //
