@@ -77,9 +77,10 @@
 #include "r_local.h"
 #include "statdump.h"
 
-
 #include "d_main.h"
 
+#include "doom_icon.c"
+
 //
 // D-DoomLoop()
 // Not a globally visible function,
@@ -543,7 +544,7 @@ void D_RunFrame()
 
     TryRunTics (); // will run at least one tic
 
-    S_UpdateSounds (players[consoleplayer].mo);// move positional sounds
+    S_UpdateSounds (players[displayplayer].mo);// move positional sounds
 
     // Update display, next frame, with current state if no profiling is on
     if (screenvisible && !nodrawers)
@@ -593,6 +594,7 @@ void D_DoomLoop (void)
     I_SetWindowTitle(gamedescription);
     I_GraphicsCheckCommandLine();
     I_SetGrabMouseCallback(D_GrabMouseCallback);
+    I_RegisterWindowIcon(doom_icon_data, doom_icon_w, doom_icon_h);
     I_InitGraphics();
     EnableLoadingDisk();
 
@@ -1777,19 +1779,25 @@ void D_DoomMain (void)
 
         // common auto-loaded files for all Doom flavors
 
-        if (gamemission < pack_chex)
+        if (gamemission < pack_chex && gamevariant != freedoom)
         {
             autoload_dir = M_GetAutoloadDir("doom-all", true);
-            DEH_AutoLoadPatches(autoload_dir);
-            W_AutoLoadWADs(autoload_dir);
-            free(autoload_dir);
+            if (autoload_dir != NULL)
+            {
+                DEH_AutoLoadPatches(autoload_dir);
+                W_AutoLoadWADs(autoload_dir);
+                free(autoload_dir);
+            }
         }
 
         // auto-loaded files per IWAD
         autoload_dir = M_GetAutoloadDir(D_SaveGameIWADName(gamemission, gamevariant), true);
-        DEH_AutoLoadPatches(autoload_dir);
-        W_AutoLoadWADs(autoload_dir);
-        free(autoload_dir);
+        if (autoload_dir != NULL)
+        {
+            DEH_AutoLoadPatches(autoload_dir);
+            W_AutoLoadWADs(autoload_dir);
+            free(autoload_dir);
+        }
     }
 
     // Load Dehacked patches specified on the command line with -deh.
@@ -1893,9 +1901,11 @@ void D_DoomMain (void)
                 while (++p != myargc && myargv[p][0] != '-')
                 {
                     char *autoload_dir;
-                    autoload_dir = M_GetAutoloadDir(M_BaseName(myargv[p]), false);
-                    W_AutoLoadWADs(autoload_dir);
-                    free(autoload_dir);
+                    if ((autoload_dir = M_GetAutoloadDir(M_BaseName(myargv[p]), false)))
+                    {
+                        W_AutoLoadWADs(autoload_dir);
+                        free(autoload_dir);
+                    }
                 }
             }
         }
@@ -1980,7 +1990,8 @@ void D_DoomMain (void)
     {
 	if (gamemode == retail &&
 	    gameversion == exe_ultimate &&
-	    gamevariant != freedoom)
+	    gamevariant != freedoom &&
+	    strncasecmp(M_BaseName(iwadfile), "rekkr", 5))
 	{
 		D_LoadSigilWad();
 	}
@@ -2033,9 +2044,11 @@ void D_DoomMain (void)
                 while (++p != myargc && myargv[p][0] != '-')
                 {
                     char *autoload_dir;
-                    autoload_dir = M_GetAutoloadDir(M_BaseName(myargv[p]), false);
-                    DEH_AutoLoadPatches(autoload_dir);
-                    free(autoload_dir);
+                    if ((autoload_dir = M_GetAutoloadDir(M_BaseName(myargv[p]), false)))
+                    {
+                        DEH_AutoLoadPatches(autoload_dir);
+                        free(autoload_dir);
+                    }
                 }
             }
         }
