@@ -594,7 +594,7 @@ void HU_Start(void)
     if (headsupactive)
 	HU_Stop();
 
-    plr = &players[consoleplayer];
+    plr = &players[displayplayer];
     message_on = false;
     message_dontfuckwithme = false;
     message_nottobefuckedwith = false;
@@ -855,7 +855,12 @@ void HU_Drawer(void)
 	HUlib_drawTextLine(&w_title, false);
     }
 
-    if (crispy->automapstats == WIDGETS_ALWAYS || (automapactive && crispy->automapstats == WIDGETS_AUTOMAP))
+    if (crispy->automapstats == WIDGETS_STBAR && (!automapactive || w_title.y != HU_TITLEY))
+    {
+	HUlib_drawTextLine(&w_kills, false);
+    }
+    else
+    if ((crispy->automapstats & WIDGETS_ALWAYS) || (automapactive && crispy->automapstats == WIDGETS_AUTOMAP))
     {
 	// [crispy] move obtrusive line out of player view
 	if (automapactive && (!crispy->automapoverlay || screenblocks < CRISPY_HUD - 1))
@@ -929,28 +934,28 @@ void HU_Erase(void)
 static void Crispy_Statsline_Ratio (char *str, int str_size, const char *prefix, int count, int total, int extra)
 {
 	if (extra)
-	    M_snprintf(str, str_size, "%s%s%s%d/%d+%d", cr_stat, prefix, crstr[CR_GRAY],
+	    M_snprintf(str, str_size, "%s%s%s%d/%d+%d ", cr_stat, prefix, crstr[CR_GRAY],
 	               count, total, extra);
 	else
-	    M_snprintf(str, str_size, "%s%s%s%d/%d", cr_stat, prefix, crstr[CR_GRAY],
+	    M_snprintf(str, str_size, "%s%s%s%d/%d ", cr_stat, prefix, crstr[CR_GRAY],
 	               count, total);
 }
 
 static void Crispy_Statsline_Remaining (char *str, int str_size, const char *prefix, int count, int total, int extra)
 {
-	M_snprintf(str, str_size, "%s%s%s%d", cr_stat, prefix, crstr[CR_GRAY],
+	M_snprintf(str, str_size, "%s%s%s%d ", cr_stat, prefix, crstr[CR_GRAY],
 		   MAX(0, total - count));
 }
 
 static void Crispy_Statsline_Percent (char *str, int str_size, const char *prefix, int count, int total, int extra)
 {
-	M_snprintf(str, str_size, "%s%s%s%d%%", cr_stat, prefix, crstr[CR_GRAY],
+	M_snprintf(str, str_size, "%s%s%s%d%% ", cr_stat, prefix, crstr[CR_GRAY],
 		   count * 100 / (total ? total : 1));
 }
 
 static void Crispy_Statsline_Boolean (char *str, int str_size, const char *prefix, int count, int total, int extra)
 {
-	M_snprintf(str, str_size, "%s%s%s%s", cr_stat, prefix, crstr[CR_GRAY],
+	M_snprintf(str, str_size, "%s%s%s%s ", cr_stat, prefix, crstr[CR_GRAY],
 		   count >= total ? "Yes" : "No");
 }
 
@@ -1064,10 +1069,35 @@ void HU_Ticker(void)
 	    w_title.y = HU_TITLEY;
     }
 
-    if (crispy->automapstats == WIDGETS_ALWAYS || (automapactive && crispy->automapstats == WIDGETS_AUTOMAP))
+    if (crispy->automapstats == WIDGETS_STBAR && (!automapactive || w_title.y != HU_TITLEY))
     {
 	crispy_statsline_func_t crispy_statsline = crispy_statslines[crispy->statsformat];
 
+	w_kills.y = HU_TITLEY;
+
+	crispy_statsline(str, sizeof(str), "K ", plr->killcount, totalkills, extrakills);
+	HUlib_clearTextLine(&w_kills);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_kills, *(s++));
+
+	crispy_statsline(str, sizeof(str), "I ", plr->itemcount, totalitems, 0);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_kills, *(s++));
+
+	crispy_statsline(str, sizeof(str), "S ", plr->secretcount, totalsecret, 0);
+	s = str;
+	while (*s)
+	    HUlib_addCharToTextLine(&w_kills, *(s++));
+    }
+    else
+    if ((crispy->automapstats & WIDGETS_ALWAYS) || (automapactive && crispy->automapstats == WIDGETS_AUTOMAP))
+    {
+	crispy_statsline_func_t crispy_statsline = crispy_statslines[crispy->statsformat];
+
+	w_kills.y = HU_MSGY + 1 * 8;
+
 	crispy_statsline(str, sizeof(str), kills, plr->killcount, totalkills, extrakills);
 	HUlib_clearTextLine(&w_kills);
 	s = str;
