@@ -141,6 +141,7 @@ multiitem_t multiitem_widgets[NUM_WIDGETS] =
     {WIDGETS_OFF, "never"},
     {WIDGETS_AUTOMAP, "in Automap"},
     {WIDGETS_ALWAYS, "always"},
+    {WIDGETS_STBAR, "status bar"},
 };
 
 multiitem_t multiitem_widescreen[NUM_RATIOS] =
@@ -388,7 +389,7 @@ void M_CrispyToggleJumping(int choice)
 void M_CrispyToggleLeveltime(int choice)
 {
     choice = 0;
-    crispy->leveltime = (crispy->leveltime + 1) % NUM_WIDGETS;
+    crispy->leveltime = (crispy->leveltime + 1) % (NUM_WIDGETS - 1);
 }
 
 void M_CrispyToggleMouseLook(int choice)
@@ -430,7 +431,7 @@ void M_CrispyTogglePitch(int choice)
 void M_CrispyTogglePlayerCoords(int choice)
 {
     choice = 0;
-    crispy->playercoords = (crispy->playercoords + 1) % (NUM_WIDGETS - 1); // [crispy] disable "always" setting
+    crispy->playercoords = (crispy->playercoords + 1) % (NUM_WIDGETS - 2); // [crispy] disable "always" setting
 }
 
 void M_CrispyToggleSecretmessage(int choice)
