@@ -1565,10 +1565,6 @@ A_PainShootSkull
 	return;
     }
 		
-    // [crispy] Lost Souls bleed Puffs
-    if (crispy->coloredblood)
-	newmobj->flags |= MF_NOBLOOD;
-
     newmobj->target = actor->target;
     A_SkullAttack (newmobj);
 }
@@ -1716,9 +1712,9 @@ static boolean CheckBossEnd(mobjtype_t motype)
                 return (gamemap == 6 && motype == MT_CYBORG)
                     || (gamemap == 8 && motype == MT_SPIDER);
 
-            // [crispy] Sigil
+            // [crispy] no trigger for auto-loaded Sigil E5
             case 5:
-                return false;
+                return (gamemap == 8 && !critical->havesigil);
 
             default:
                 return gamemap == 8;
