@@ -102,9 +102,14 @@ void P_LoadSegs_DeePBSP (int lump)
 	mapseg_deepbsp_t *ml = data + i;
 	int side, linedef;
 	line_t *ldef;
+	int vn1, vn2;
 
-	li->v1 = &vertexes[ml->v1];
-	li->v2 = &vertexes[ml->v2];
+	// [MB] 2020-04-30: Fix endianess for DeePBSDP V4 nodes
+	vn1 = LONG(ml->v1);
+	vn2 = LONG(ml->v2);
+
+	li->v1 = &vertexes[vn1];
+	li->v2 = &vertexes[vn2];
 
 	li->angle = (SHORT(ml->angle))<<FRACBITS;
 
@@ -114,12 +119,17 @@ void P_LoadSegs_DeePBSP (int lump)
 	li->linedef = ldef;
 	side = SHORT(ml->side);
 
-        // e6y: check for wrong indexes
-        if ((unsigned)ldef->sidenum[side] >= (unsigned)numsides)
-        {
-            I_Error("P_LoadSegs: linedef %d for seg %d references a non-existent sidedef %d",
-                    linedef, i, (unsigned)ldef->sidenum[side]);
-        }
+	// e6y: check for wrong indexes
+	if ((unsigned)linedef >= (unsigned)numlines)
+	{
+		I_Error("P_LoadSegs: seg %d references a non-existent linedef %d",
+			i, (unsigned)linedef);
+	}
+	if ((unsigned)ldef->sidenum[side] >= (unsigned)numsides)
+	{
+		I_Error("P_LoadSegs: linedef %d for seg %d references a non-existent sidedef %d",
+			linedef, i, (unsigned)ldef->sidenum[side]);
+	}
 
 	li->sidedef = &sides[ldef->sidenum[side]];
 	li->frontsector = sides[ldef->sidenum[side]].sector;
@@ -167,8 +177,9 @@ void P_LoadSubsectors_DeePBSP (int lump)
 
     for (i = 0; i < numsubsectors; i++)
     {
-	subsectors[i].numlines = (int)data[i].numsegs;
-	subsectors[i].firstline = (int)data[i].firstseg;
+	// [MB] 2020-04-30: Fix endianess for DeePBSDP V4 nodes
+	subsectors[i].numlines = (unsigned short)SHORT(data[i].numsegs);
+	subsectors[i].firstline = LONG(data[i].firstseg);
     }
 
     W_ReleaseLumpNum(lump);
@@ -210,7 +221,8 @@ void P_LoadNodes_DeePBSP (int lump)
 	for (j = 0; j < 2; j++)
 	{
 	    int k;
-	    no->children[j] = (unsigned int)(mn->children[j]);
+	    // [MB] 2020-04-30: Fix endianess for DeePBSDP V4 nodes
+	    no->children[j] = LONG(mn->children[j]);
 
 	    for (k = 0; k < 4; k++)
 		no->bbox[j][k] = SHORT(mn->bbox[j][k])<<FRACBITS;
@@ -228,6 +240,7 @@ void P_LoadNodes_DeePBSP (int lump)
 // - inlined P_LoadZSegs()
 // - added support for compressed ZDBSP nodes
 // - added support for flipped levels
+// [MB] 2020-04-30: Fix endianess for ZDoom extended nodes
 void P_LoadNodes_ZDBSP (int lump, boolean compressed)
 {
     byte *data;
@@ -305,10 +318,10 @@ void P_LoadNodes_ZDBSP (int lump, boolean compressed)
 
     // 1. Load new vertices added during node building
 
-    orgVerts = *((unsigned int*)data);
+    orgVerts = LONG(*((unsigned int*)data));
     data += sizeof(orgVerts);
 
-    newVerts = *((unsigned int*)data);
+    newVerts = LONG(*((unsigned int*)data));
     data += sizeof(newVerts);
 
     if (orgVerts + newVerts == (unsigned int)numvertexes)
@@ -325,11 +338,11 @@ void P_LoadNodes_ZDBSP (int lump, boolean compressed)
     for (i = 0; i < newVerts; i++)
     {
 	newvertarray[i + orgVerts].r_x =
-	newvertarray[i + orgVerts].x = *((unsigned int*)data);
+	newvertarray[i + orgVerts].x = LONG(*((unsigned int*)data));
 	data += sizeof(newvertarray[0].x);
 
 	newvertarray[i + orgVerts].r_y =
-	newvertarray[i + orgVerts].y = *((unsigned int*)data);
+	newvertarray[i + orgVerts].y = LONG(*((unsigned int*)data));
 	data += sizeof(newvertarray[0].y);
     }
 
@@ -348,7 +361,7 @@ void P_LoadNodes_ZDBSP (int lump, boolean compressed)
 
     // 2. Load subsectors
 
-    numSubs = *((unsigned int*)data);
+    numSubs = LONG(*((unsigned int*)data));
     data += sizeof(numSubs);
 
     if (numSubs < 1)
@@ -362,15 +375,15 @@ void P_LoadNodes_ZDBSP (int lump, boolean compressed)
 	mapsubsector_zdbsp_t *mseg = (mapsubsector_zdbsp_t*) data + i;
 
 	subsectors[i].firstline = currSeg;
-	subsectors[i].numlines = mseg->numsegs;
-	currSeg += mseg->numsegs;
+	subsectors[i].numlines = LONG(mseg->numsegs);
+	currSeg += LONG(mseg->numsegs);
     }
 
     data += numsubsectors * sizeof(mapsubsector_zdbsp_t);
 
     // 3. Load segs
 
-    numSegs = *((unsigned int*)data);
+    numSegs = LONG(*((unsigned int*)data));
     data += sizeof(numSegs);
 
     // The number of stored segs should match the number of segs used by subsectors
@@ -389,21 +402,29 @@ void P_LoadNodes_ZDBSP (int lump, boolean compressed)
 	unsigned char side;
 	seg_t *li = segs + i;
 	mapseg_zdbsp_t *ml = (mapseg_zdbsp_t *) data + i;
+	unsigned int v1, v2;
 
-	li->v1 = &vertexes[ml->v1];
-	li->v2 = &vertexes[ml->v2];
+	v1 = LONG(ml->v1);
+	v2 = LONG(ml->v2);
+	li->v1 = &vertexes[v1];
+	li->v2 = &vertexes[v2];
 
 	linedef = (unsigned short)SHORT(ml->linedef);
 	ldef = &lines[linedef];
 	li->linedef = ldef;
 	side = ml->side;
 
-        // e6y: check for wrong indexes
-        if ((unsigned)ldef->sidenum[side] >= (unsigned)numsides)
-        {
-            I_Error("P_LoadSegs: linedef %d for seg %d references a non-existent sidedef %d",
-                    linedef, i, (unsigned)ldef->sidenum[side]);
-        }
+	// e6y: check for wrong indexes
+	if ((unsigned)linedef >= (unsigned)numlines)
+	{
+		I_Error("P_LoadSegs: seg %d references a non-existent linedef %d",
+			i, (unsigned)linedef);
+	}
+	if ((unsigned)ldef->sidenum[side] >= (unsigned)numsides)
+	{
+		I_Error("P_LoadSegs: linedef %d for seg %d references a non-existent sidedef %d",
+			linedef, i, (unsigned)ldef->sidenum[side]);
+	}
 
 	li->sidedef = &sides[ldef->sidenum[side]];
 	li->frontsector = sides[ldef->sidenum[side]].sector;
@@ -437,7 +458,7 @@ void P_LoadNodes_ZDBSP (int lump, boolean compressed)
 
     // 4. Load nodes
 
-    numNodes = *((unsigned int*)data);
+    numNodes = LONG(*((unsigned int*)data));
     data += sizeof(numNodes);
 
     numnodes = numNodes;
@@ -456,8 +477,8 @@ void P_LoadNodes_ZDBSP (int lump, boolean compressed)
 
 	for (j = 0; j < 2; j++)
 	{
-	    no->children[j] = (unsigned int)(mn->children[j]);
-
+	    no->children[j] = LONG(mn->children[j]);
+	
 	    for (k = 0; k < 4; k++)
 		no->bbox[j][k] = SHORT(mn->bbox[j][k])<<FRACBITS;
 	}
