@@ -56,6 +56,9 @@ char *P_TempSaveGameFile(void)
 
 // Get the filename of the save game file to use for the specified slot.
 
+// [FG] support up to 8 pages of savegames
+extern int savepage;
+
 char *P_SaveGameFile(int slot)
 {
     static char *filename = NULL;
@@ -68,7 +71,7 @@ char *P_SaveGameFile(int slot)
         filename = malloc(filename_size);
     }
 
-    DEH_snprintf(basename, 32, SAVEGAMENAME "%d.dsg", slot);
+    DEH_snprintf(basename, 32, SAVEGAMENAME "%d.dsg", 10*savepage+slot);
     M_snprintf(filename, filename_size, "%s%s", savegamedir, basename);
 
     return filename;
@@ -415,6 +418,13 @@ static void saveg_read_mobj_t(mobj_t *str)
 
     // struct mobj_s* tracer;
     str->tracer = saveg_readp();
+
+    // [crispy] new mobj_t fields used for interpolation
+    str->interp = 0;
+    str->oldx = 0;
+    str->oldy = 0;
+    str->oldz = 0;
+    str->oldangle = 0;
 }
 
 // [crispy] enumerate all thinker pointers
