@@ -16,6 +16,8 @@
 
 // P_Spec.c
 
+#include <stdlib.h> // [crispy] free()
+
 #include "doomdef.h"
 #include "deh_str.h"
 #include "i_system.h"
@@ -46,7 +48,9 @@ typedef enum
 
 // Data
 
-int *LevelAmbientSfx[MAX_AMBIENT_SFX];
+// [crispy] remove the ambient sound limit
+static int **LevelAmbientSfx = NULL;
+static int AmbSfxMax = 0;
 int *AmbSfxPtr;
 int AmbSfxCount;
 int AmbSfxTics;
@@ -1260,12 +1264,24 @@ void P_SpawnSpecials(void)
 //
 //----------------------------------------------------------------------------
 
+// [crispy] fix ambient sounds stop playing
+static int (*Amb_Random)(void);
+
 void P_InitAmbientSound(void)
 {
     AmbSfxCount = 0;
     AmbSfxVolume = 0;
     AmbSfxTics = 10 * TICRATE;
     AmbSfxPtr = AmbSndSeqInit;
+    // [crispy] remove the ambient sound limit
+    AmbSfxMax = 0;
+    if (LevelAmbientSfx)
+    {
+        free(LevelAmbientSfx);
+        LevelAmbientSfx = NULL;
+    }
+    // [crispy] fix ambient sounds stop playing
+    Amb_Random = P_Random;
 }
 
 //----------------------------------------------------------------------------
@@ -1278,9 +1294,11 @@ void P_InitAmbientSound(void)
 
 void P_AddAmbientSfx(int sequence)
 {
-    if (AmbSfxCount == MAX_AMBIENT_SFX)
+    // [crispy] remove the ambient sound limit
+    if (AmbSfxCount >= AmbSfxMax)
     {
-        I_Error("Too many ambient sound sequences");
+        AmbSfxMax = AmbSfxMax ? AmbSfxMax * 2 : MAX_AMBIENT_SFX;
+        LevelAmbientSfx = I_Realloc(LevelAmbientSfx, sizeof(*LevelAmbientSfx) * AmbSfxMax);
     }
     LevelAmbientSfx[AmbSfxCount++] = AmbientSfx[sequence];
 }
@@ -1314,7 +1332,7 @@ void P_AmbientSound(void)
         switch (cmd)
         {
             case afxcmd_play:
-                AmbSfxVolume = P_Random() >> 2;
+                AmbSfxVolume = Amb_Random() >> 2;
                 S_StartSoundAtVolume(NULL, *AmbSfxPtr++, AmbSfxVolume);
                 break;
             case afxcmd_playabsvol:
@@ -1340,12 +1358,18 @@ void P_AmbientSound(void)
                 done = true;
                 break;
             case afxcmd_delayrand:
-                AmbSfxTics = P_Random() & (*AmbSfxPtr++);
+                AmbSfxTics = Amb_Random() & (*AmbSfxPtr++);
+                // [crispy] fix ambient sounds stop playing
+                if (AmbSfxTics == 0)
+                {
+                    AmbSfxTics++;
+                    Amb_Random = Crispy_Random;
+                }
                 done = true;
                 break;
             case afxcmd_end:
-                AmbSfxTics = 6 * TICRATE + P_Random();
-                AmbSfxPtr = LevelAmbientSfx[P_Random() % AmbSfxCount];
+                AmbSfxTics = 6 * TICRATE + Amb_Random();
+                AmbSfxPtr = LevelAmbientSfx[Amb_Random() % AmbSfxCount];
                 done = true;
                 break;
             default:
