@@ -48,6 +48,9 @@
 #define	NUMCOLORMAPS		32      // number of diminishing
 #define	INVERSECOLORMAP		32
 
+#define LOOKDIRMIN 110 // [crispy] -110, actually
+#define LOOKDIRMAX 90
+#define LOOKDIRS (LOOKDIRMIN + 1 + LOOKDIRMAX) // [crispy] lookdir range: -110..0..90
 /*
 ==============================================================================
 
@@ -227,7 +230,9 @@ typedef struct vissprite_s
     fixed_t xiscale;            // negative if flipped
     fixed_t texturemid;
     int patch;
-    lighttable_t *colormap;
+    // [crispy] brightmaps for select sprites
+    lighttable_t *colormap[2];
+    const byte *brightmap;
     int mobjflags;              // for color translation and shadow draw
     boolean psprite;            // true if psprite
     fixed_t footclip;           // foot clipping
@@ -388,7 +393,8 @@ extern int *lastopening;    // [crispy] 32-bit integer math
 extern int floorclip[MAXWIDTH];   // [crispy] 32-bit integer math
 extern int ceilingclip[MAXWIDTH]; // [crispy] 32-bit integer math
 
-extern fixed_t yslope[MAXHEIGHT];
+extern fixed_t *yslope;
+extern fixed_t yslopes[LOOKDIRS][MAXHEIGHT]; // [crispy]
 extern fixed_t distscale[MAXWIDTH];
 
 void R_InitPlanes(void);
@@ -469,7 +475,7 @@ void R_ClipVisSprite(vissprite_t * vis, int xl, int xh);
 //
 //=============================================================================
 
-extern lighttable_t *dc_colormap;
+extern lighttable_t *dc_colormap[2];  // [crispy] brightmaps
 extern int dc_x;
 extern int dc_yl;
 extern int dc_yh;
@@ -477,6 +483,7 @@ extern fixed_t dc_iscale;
 extern fixed_t dc_texturemid;
 extern int dc_texheight;
 extern byte *dc_source;         // first pixel in a column
+extern const byte *dc_brightmap;  // [crispy] brightmaps
 
 void R_DrawColumn(void);
 void R_DrawColumnLow(void);
@@ -489,12 +496,13 @@ void R_DrawTranslatedColumnLow(void);
 extern int ds_y;
 extern int ds_x1;
 extern int ds_x2;
-extern lighttable_t *ds_colormap;
+extern lighttable_t *ds_colormap[2];  // [crispy] brightmaps
 extern fixed_t ds_xfrac;
 extern fixed_t ds_yfrac;
 extern fixed_t ds_xstep;
 extern fixed_t ds_ystep;
 extern byte *ds_source;         // start of a 64*64 tile image
+extern const byte *ds_brightmap;  // [crispy] brightmaps
 
 extern byte *translationtables;
 extern byte *dc_translation;
