@@ -19,6 +19,7 @@
 #include "i_system.h"
 #include "i_swap.h"
 #include "m_misc.h"
+#include "r_bmaps.h"
 #include "r_local.h"
 #include "p_local.h"
 #include "v_trans.h" // [crispy] color translation and color string tables
@@ -56,6 +57,7 @@ int *texturecompositesize;
 short **texturecolumnlump;
 unsigned short **texturecolumnofs;
 byte **texturecomposite;
+const byte **texturebrightmap;  // [crispy] brightmaps
 
 int *flattranslation;           // for global animation
 int *texturetranslation;        // for global animation
@@ -344,6 +346,7 @@ void R_InitTextures(void)
     texturecompositesize = Z_Malloc(numtextures * sizeof(int), PU_STATIC, 0);
     texturewidthmask = Z_Malloc(numtextures * sizeof(int), PU_STATIC, 0);
     textureheight = Z_Malloc(numtextures * sizeof(fixed_t), PU_STATIC, 0);
+    texturebrightmap = Z_Malloc (numtextures * sizeof(*texturebrightmap), PU_STATIC, 0);
 
     for (i = 0; i < numtextures; i++, directory++)
     {
@@ -369,6 +372,8 @@ void R_InitTextures(void)
         memcpy(texture->name, mtexture->name, sizeof(texture->name));
         mpatch = &mtexture->patches[0];
         patch = &texture->patches[0];
+        // [crispy] initialize brightmaps
+        texturebrightmap[i] = R_BrightmapForTexName(texture->name);
         for (j = 0; j < texture->patchcount; j++, mpatch++, patch++)
         {
             patch->originx = SHORT(mpatch->originx);
