@@ -63,6 +63,10 @@ lighttable_t *scalelight[LIGHTLEVELS][MAXLIGHTSCALE];
 lighttable_t *scalelightfixed[MAXLIGHTSCALE];
 lighttable_t *zlight[LIGHTLEVELS][MAXLIGHTZ];
 
+// [AM] Fractional part of the current tic, in the half-open
+//      range of [0.0, 1.0).  Used for interpolation.
+extern fixed_t          fractionaltic;
+
 int extralight;                 // bumped light from gun blasts
 
 void (*colfunc) (void);
@@ -363,6 +367,26 @@ fixed_t R_ScaleFromGlobalAngle(angle_t visangle)
 }
 
 
+// [AM] Interpolate between two angles.
+angle_t R_InterpolateAngle(angle_t oangle, angle_t nangle, fixed_t scale)
+{
+    if (nangle == oangle)
+        return nangle;
+    else if (nangle > oangle)
+    {
+        if (nangle - oangle < ANG270)
+            return oangle + (angle_t)((nangle - oangle) * FIXED2DOUBLE(scale));
+        else // Wrapped around
+            return oangle - (angle_t)((oangle - nangle) * FIXED2DOUBLE(scale));
+    }
+    else // nangle < oangle
+    {
+        if (oangle - nangle < ANG270)
+            return oangle - (angle_t)((oangle - nangle) * FIXED2DOUBLE(scale));
+        else // Wrapped around
+            return oangle + (angle_t)((nangle - oangle) * FIXED2DOUBLE(scale));
+    }
+}
 
 /*
 =================
@@ -575,7 +599,7 @@ void R_ExecuteSetViewSize(void)
 	{
 		const int widescreen_edge_aligner = (16 << crispy->hires) - 1;
 
-		scaledviewwidth = viewheight*SCREENWIDTH/(SCREENHEIGHT-(42<<crispy->hires));
+		scaledviewwidth = viewheight*SCREENWIDTH/(SCREENHEIGHT-SBARHEIGHT);
 		// [crispy] make sure scaledviewwidth is an integer multiple of the bezel patch width
 		scaledviewwidth = (scaledviewwidth + widescreen_edge_aligner) & (int)~widescreen_edge_aligner;
 		scaledviewwidth = MIN(scaledviewwidth, SCREENWIDTH);
@@ -585,8 +609,6 @@ void R_ExecuteSetViewSize(void)
 		scaledviewwidth = scaledviewwidth_nonwide;
 	}
     }
-    // [crispy] make sure viewheight is always an even number
-    viewheight &= ~1;
 
     detailshift = setdetail;
     viewwidth = scaledviewwidth >> detailshift;
@@ -635,10 +657,19 @@ void R_ExecuteSetViewSize(void)
 //
     for (i = 0; i < viewheight; i++)
     {
-        dy = ((i - viewheight / 2) << FRACBITS) + FRACUNIT / 2;
+        // [crispy] re-generate lookup-table for yslope[] (free look)
+        // whenever "detailshift" or "screenblocks" change
+        const fixed_t num = (viewwidth_nonwide<<detailshift)/2*FRACUNIT;
+        for (j = 0; j < LOOKDIRS; j++)
+        {
+        dy = ((i - (viewheight / 2 + ((j - LOOKDIRMIN) * (1 << crispy->hires)) *
+                (screenblocks < 11 ? screenblocks : 11) / 10)) << FRACBITS) +
+                FRACUNIT / 2;
         dy = abs(dy);
-        yslope[i] = FixedDiv((viewwidth_nonwide << detailshift) / 2 * FRACUNIT, dy);
+        yslopes[j][i] = FixedDiv(num, dy);
+        }
     }
+    yslope = yslopes[LOOKDIRMIN];
 
     for (i = 0; i < viewwidth; i++)
     {
@@ -741,39 +772,73 @@ void R_SetupFrame(player_t * player)
     int tableAngle;
     int tempCentery;
     int intensity;
+    static int x_quake, y_quake, quaketime; // [crispy]
+    int pitch; // [crispy]
 
     //drawbsp = 1;
     viewplayer = player;
     // haleyjd: removed WATCOMC
     // haleyjd FIXME: viewangleoffset handling?
-    viewangle = player->mo->angle + viewangleoffset;
-    tableAngle = viewangle >> ANGLETOFINESHIFT;
-    viewx = player->mo->x;
-    viewy = player->mo->y;
+    if (crispy->uncapped && leveltime > 1 && player->mo->interp == true && leveltime > oldleveltime)
+    {
+        viewx = player->mo->oldx + FixedMul(player->mo->x - player->mo->oldx, fractionaltic);
+        viewy = player->mo->oldy + FixedMul(player->mo->y - player->mo->oldy, fractionaltic);
+        viewz = player->oldviewz + FixedMul(player->viewz - player->oldviewz, fractionaltic);
+        viewangle = R_InterpolateAngle(player->mo->oldangle, player->mo->angle, fractionaltic) + viewangleoffset;
+        pitch = player->oldlookdir + (player->lookdir - player->oldlookdir) *
+                FIXED2DOUBLE(fractionaltic);
+    }
+    else
+    {
+        viewangle = player->mo->angle + viewangleoffset;
+        viewx = player->mo->x;
+        viewy = player->mo->y;
+        viewz = player->viewz;
+        pitch = player->lookdir; // [crispy]
+    }
 
     if (localQuakeHappening[displayplayer] && !paused)
     {
-        intensity = localQuakeHappening[displayplayer];
-        viewx += ((M_Random() % (intensity << 2))
-                  - (intensity << 1)) << FRACBITS;
-        viewy += ((M_Random() % (intensity << 2))
-                  - (intensity << 1)) << FRACBITS;
+        // [crispy] only get new quake values once every gametic
+        if (leveltime > quaketime)
+        {
+            intensity = localQuakeHappening[displayplayer];
+            x_quake = ((M_Random() % (intensity << 2))
+                           - (intensity << 1)) << FRACBITS;
+            y_quake = ((M_Random() % (intensity << 2))
+                           - (intensity << 1)) << FRACBITS;
+            quaketime = leveltime;
+        }
+
+        if (crispy->uncapped)
+        {
+            viewx += FixedMul(x_quake, fractionaltic);
+            viewy += FixedMul(y_quake, fractionaltic);
+        }
+        else
+        {
+            viewx += x_quake;
+            viewy += y_quake;
+        }
+    }
+    else if (!localQuakeHappening[displayplayer])
+    {
+        quaketime = 0;
     }
 
     extralight = player->extralight;
-    viewz = player->viewz;
 
-    tempCentery = viewheight / 2 + ((player->lookdir) << crispy->hires) * screenblocks / 10;
+    tableAngle = viewangle >> ANGLETOFINESHIFT;
+
+    // [crispy] apply new yslope[] whenever "lookdir", "detailshift" or
+    // "screenblocks" change
+    tempCentery = viewheight / 2 + (pitch * (1 << crispy->hires)) *
+                    (screenblocks < 11 ? screenblocks : 11) / 10;
     if (centery != tempCentery)
     {
         centery = tempCentery;
         centeryfrac = centery << FRACBITS;
-        for (i = 0; i < viewheight; i++)
-        {
-            yslope[i] = FixedDiv((viewwidth_nonwide << detailshift) / 2 * FRACUNIT,
-                                 abs(((i - centery) << FRACBITS) +
-                                     FRACUNIT / 2));
-        }
+        yslope = yslopes[LOOKDIRMIN + pitch];
     }
     viewsin = finesine[tableAngle];
     viewcos = finecosine[tableAngle];
@@ -833,12 +898,25 @@ void R_SetupFrame(player_t * player)
 
 void R_RenderPlayerView(player_t * player)
 {
+    extern void PO_InterpolatePolyObjects(void);
+    extern void R_InterpolateTextureOffset(void);
+    extern boolean automapactive;
+
     R_SetupFrame(player);
     R_ClearClipSegs();
     R_ClearDrawSegs();
     R_ClearPlanes();
     R_ClearSprites();
+
+    if (automapactive && !crispy->automapoverlay)
+    {
+        R_RenderBSPNode(numnodes - 1);
+        return;
+    }
+
     NetUpdate();                // check for new console commands
+    PO_InterpolatePolyObjects(); // [crispy] Interpolate polyobjects here
+    R_InterpolateTextureOffset(); // [crispy] Smooth texture scrolling
 
     // Make displayed player invisible locally
     if (localQuakeHappening[displayplayer] && gamestate == GS_LEVEL)
