@@ -802,7 +802,8 @@ void SB_Drawer(void)
         DrawSoundInfo();
     }
     CPlayer = &players[consoleplayer];
-    if (viewheight == SCREENHEIGHT && !automapactive)
+    if (viewheight == SCREENHEIGHT
+        && !(automapactive && !crispy->automapoverlay))
     {
         DrawFullScreenStuff();
         SB_state = -1;
@@ -812,7 +813,19 @@ void SB_Drawer(void)
         if (SB_state == -1)
         {
             RefreshBackground(); // [crispy] for widescreen
-            V_DrawPatch(0, 134, PatchH2BAR);
+
+            // [crispy] support wide status bars with 0 offset
+            if (SHORT(PatchH2BAR->width) > ORIGWIDTH &&
+                    SHORT(PatchH2BAR->leftoffset) == 0)
+            {
+                V_DrawPatch((ORIGWIDTH - SHORT(PatchH2BAR->width)) / 2, 134,
+                        PatchH2BAR);
+            }
+            else
+            {
+                V_DrawPatch(0, 134, PatchH2BAR);
+            }
+
             oldhealth = -1;
         }
         DrawCommonBar();
@@ -821,7 +834,7 @@ void SB_Drawer(void)
             if (SB_state != 0)
             {
                 // Main interface
-                if (!automapactive)
+                if (!(automapactive && !crispy->automapoverlay))
                 {
                     V_DrawPatch(38, 162, PatchSTATBAR);
                 }
@@ -839,7 +852,7 @@ void SB_Drawer(void)
                 oldweapon = -1;
                 oldkeys = -1;
             }
-            if (!automapactive)
+            if (!(automapactive && !crispy->automapoverlay))
             {
                 DrawMainBar();
             }
@@ -869,10 +882,13 @@ static void DrawAnimatedIcons(void)
 {
     int frame;
     static boolean hitCenterFrame;
+    int spinfly_x, spinspeed_x, spindefense_x, spinminotaur_x; // [crispy]
 
     // Wings of wrath
     if (CPlayer->powers[pw_flight])
     {
+        spinfly_x = 20 - WIDESCREENDELTA; // [crispy]
+
         if (CPlayer->powers[pw_flight] > BLINKTHRESHOLD
             || !(CPlayer->powers[pw_flight] & 16))
         {
@@ -881,13 +897,15 @@ static void DrawAnimatedIcons(void)
             {
                 if (hitCenterFrame && (frame != 15 && frame != 0))
                 {
-                    V_DrawPatch(20, 19, W_CacheLumpNum(SpinFlylump + 15,
-                                                       PU_CACHE));
+                    V_DrawPatch(spinfly_x, 19,
+                                W_CacheLumpNum(SpinFlylump + 15,
+                                                PU_CACHE));
                 }
                 else
                 {
-                    V_DrawPatch(20, 19, W_CacheLumpNum(SpinFlylump + frame,
-                                                       PU_CACHE));
+                    V_DrawPatch(spinfly_x, 19,
+                                W_CacheLumpNum(SpinFlylump + frame,
+                                                PU_CACHE));
                     hitCenterFrame = false;
                 }
             }
@@ -895,14 +913,16 @@ static void DrawAnimatedIcons(void)
             {
                 if (!hitCenterFrame && (frame != 15 && frame != 0))
                 {
-                    V_DrawPatch(20, 19, W_CacheLumpNum(SpinFlylump + frame,
-                                                       PU_CACHE));
+                    V_DrawPatch(spinfly_x, 19,
+                                W_CacheLumpNum(SpinFlylump + frame,
+                                                PU_CACHE));
                     hitCenterFrame = false;
                 }
                 else
                 {
-                    V_DrawPatch(20, 19, W_CacheLumpNum(SpinFlylump + 15,
-                                                       PU_CACHE));
+                    V_DrawPatch(spinfly_x, 19,
+                                W_CacheLumpNum(SpinFlylump + 15,
+                                                PU_CACHE));
                     hitCenterFrame = true;
                 }
             }
@@ -914,12 +934,15 @@ static void DrawAnimatedIcons(void)
     // Speed Boots
     if (CPlayer->powers[pw_speed])
     {
+        spinspeed_x = 60 - WIDESCREENDELTA; // [crispy]
+
         if (CPlayer->powers[pw_speed] > BLINKTHRESHOLD
             || !(CPlayer->powers[pw_speed] & 16))
         {
             frame = (leveltime / 3) & 15;
-            V_DrawPatch(60, 19, W_CacheLumpNum(SpinSpeedLump + frame,
-                                               PU_CACHE));
+            V_DrawPatch(spinspeed_x, 19,
+                        W_CacheLumpNum(SpinSpeedLump + frame,
+                                        PU_CACHE));
         }
         BorderTopRefresh = true;
         UpdateState |= I_MESSAGES;
@@ -928,12 +951,15 @@ static void DrawAnimatedIcons(void)
     // Defensive power
     if (CPlayer->powers[pw_invulnerability])
     {
+        spindefense_x = 260 + WIDESCREENDELTA; // [crispy]
+
         if (CPlayer->powers[pw_invulnerability] > BLINKTHRESHOLD
             || !(CPlayer->powers[pw_invulnerability] & 16))
         {
             frame = (leveltime / 3) & 15;
-            V_DrawPatch(260, 19, W_CacheLumpNum(SpinDefenseLump + frame,
-                                                PU_CACHE));
+            V_DrawPatch(spindefense_x, 19,
+                        W_CacheLumpNum(SpinDefenseLump + frame,
+                                        PU_CACHE));
         }
         BorderTopRefresh = true;
         UpdateState |= I_MESSAGES;
@@ -942,12 +968,15 @@ static void DrawAnimatedIcons(void)
     // Minotaur Active
     if (CPlayer->powers[pw_minotaur])
     {
+        spinminotaur_x = 300 + WIDESCREENDELTA; // [crispy]
+
         if (CPlayer->powers[pw_minotaur] > BLINKTHRESHOLD
             || !(CPlayer->powers[pw_minotaur] & 16))
         {
             frame = (leveltime / 3) & 15;
-            V_DrawPatch(300, 19, W_CacheLumpNum(SpinMinotaurLump + frame,
-                                                PU_CACHE));
+            V_DrawPatch(spinminotaur_x, 19,
+                        W_CacheLumpNum(SpinMinotaurLump + frame,
+                                        PU_CACHE));
         }
         BorderTopRefresh = true;
         UpdateState |= I_MESSAGES;
