@@ -128,10 +128,15 @@ int mousebuse = -1;
 int mousebmouselook = -1; // [crispy]
 int mousebreverse = -1; // [crispy]
 
+int mousebmapzoomin = 3; // [crispy]
+int mousebmapzoomout = 4; // [crispy]
+int mousebmapmaxzoom = -1; // [crispy]
+int mousebmapfollow = -1; // [crispy]
 int mousebprevweapon = 4; // [crispy]
 int mousebnextweapon = 3; // [crispy]
 int mousebinvleft = -1;
 int mousebinvright = -1;
+int mousebuseartifact = -1;
 
 int key_message_refresh = KEY_ENTER;
 int key_pause = KEY_PAUSE;
@@ -309,6 +314,7 @@ void M_BindHereticControls(void)
 
     M_BindIntVariable("mouseb_invleft", &mousebinvleft);
     M_BindIntVariable("mouseb_invright", &mousebinvright);
+    M_BindIntVariable("mouseb_useartifact", &mousebuseartifact);
 
     M_BindIntVariable("key_arti_quartz",        &key_arti_quartz);
     M_BindIntVariable("key_arti_urn",           &key_arti_urn);
@@ -408,6 +414,10 @@ void M_BindMapControls(void)
     M_BindIntVariable("key_map_clearmark",  &key_map_clearmark);
     M_BindIntVariable("key_map_overlay",    &key_map_overlay); // [crispy]
     M_BindIntVariable("key_map_rotate",     &key_map_rotate); // [crispy]
+    M_BindIntVariable("mouseb_mapzoomin",   &mousebmapzoomin); // [crispy]
+    M_BindIntVariable("mouseb_mapzoomout",  &mousebmapzoomout); // [crispy]
+    M_BindIntVariable("mouseb_mapmaxzoom",  &mousebmapmaxzoom); // [crispy]
+    M_BindIntVariable("mouseb_mapfollow",   &mousebmapfollow); // [crispy]
 }
 
 void M_BindMenuControls(void)
