@@ -167,6 +167,12 @@ extern int mousebprevweapon;
 extern int mousebnextweapon;
 extern int mousebinvleft;
 extern int mousebinvright;
+extern int mousebuseartifact;
+
+extern int mousebmapzoomin; // [crispy]
+extern int mousebmapzoomout; // [crispy]
+extern int mousebmapmaxzoom; // [crispy]
+extern int mousebmapfollow; // [crispy]
 
 extern int joybfire;
 extern int joybstrafe;
