@@ -71,7 +71,7 @@ static int *shortcuts[] = { &key_menu_help, &key_menu_save, &key_menu_load,
                             &key_menu_screenshot, &key_menu_cleanscreenshot,
                             &key_message_refresh, &key_multi_msg,
                             &key_multi_msgplayer[0], &key_multi_msgplayer[1],
-                            &key_multi_msgplayer[2], &key_multi_msgplayer[3] };
+                            &key_multi_msgplayer[2], &key_multi_msgplayer[3], NULL };
 
 static int *map_keys[] = { &key_map_north, &key_map_south, &key_map_east,
                            &key_map_west, &key_map_zoomin, &key_map_zoomout,
@@ -230,7 +230,7 @@ static void ConfigExtraKeys(TXT_UNCAST_ARG(widget), TXT_UNCAST_ARG(unused))
         AddKeyControl(table, "Toggle vert. mouse", &key_togglenovert);
         AddKeyControl(table, "Quick Reverse", &key_reverse);
         }
-        else if (gamemission == heretic)
+        else if (gamemission == heretic || gamemission == hexen)
         {
         AddSectionLabel(table, "View", false);
 
