@@ -557,7 +557,7 @@ void M_ReadSaveStrings(void)
             Z_Free(fname);
         fname = M_SafeFilePath(savegamedir, M_MakeStrifeSaveDir(i, "\\name"));
 
-        handle = fopen(fname, "rb");
+        handle = M_fopen(fname, "rb");
         if(handle == NULL)
         {
             M_StringCopy(savegamestrings[i], DEH_String(EMPTYSTRING),
